#!/usr/bin/python3

# import modules or classes for controlling components and code from code libraries
# import the PMA Light Sensor class from the ptpma.components library (PMA = pi-top Maker Architecture)
# sleep lets us tell our components to wait (on or off) for a specified amount of time
from ptpma.components import PMALightSensor
from ptpma.components import PMAUltrasonicSensor
from ptpma.components import PMALed

import RPi.GPIO as GPIO
import os
import sys
import numpy

#import pigpio

from time import sleep

# +++++++++++++++++++ Sub motor control ++++++++++++++++++++++++++++++++
def motors(motorR, motorL, timeStep):
  print("Hello from a function: motorR=%7.2f" % motorR, ", motorL=%7.2f" % motorL)

  motorMAX = abs(motorL)
  if(abs(motorR) > motorMAX):
    motorMAX = abs(motorR)

  timeStepL = timeStep * abs(motorL) / motorMAX
  timeStepR = timeStep * abs(motorR) / motorMAX
  print("timeStepR = ", timeStepR, ", timeStep=", timeStep, ", timeStepL = ", timeStepL, ", motorR=", motorR, ", motorL=", motorL)
  for i in numpy.arange(0, motorMAX, timeStep):
    if(motorR > 0):
      drive_R_pwm.ChangeDutyCycle(5.0)
    if(motorR < 0):
      drive_R_pwm.ChangeDutyCycle(10.0)
    if(motorL > 0):
      drive_L_pwm.ChangeDutyCycle(10.0)
    if(motorL < 0):
      drive_L_pwm.ChangeDutyCycle(5.0)
    if(timeStepR > timeStepL):
      sleep(timeStepL)
      drive_L_pwm.ChangeDutyCycle(0.0)
      sleep(timeStep-timeStepL)
    elif(timeStepR < timeStepL):
      sleep(timeStepR)
      drive_R_pwm.ChangeDutyCycle(0.0)
      sleep(timeStep-timeStepR)
    else:
      sleep(timeStep)

  drive_R_pwm.ChangeDutyCycle(0.0)
  drive_L_pwm.ChangeDutyCycle(0.0)

# ----------------End of Sub motor control -----------------------------




# name the component(s) and state which port(s) they are connected to
light_sensor = PMALightSensor("A0")
ultrasonic_sensor = PMAUltrasonicSensor("D5")

led_1 = PMALed("D0")
led_2 = PMALed("D6")
led_3 = PMALed("D2") # White LEDs, inverse logic !!!
led_4 = PMALed("D1")
led_5 = PMALed("D7")

cameraServo = 21
drive_R = 20
drive_L = 16

#pi = pigpio.pi() # Works only with pin 18, troubles sound
#    pi.hardware_PWM(18, 50, 50000) # pin, frequency, dutycycle (1000000 = 100%)
#    pi.hardware_PWM(18, 50, 100000)

GPIO.setup(cameraServo, GPIO.OUT)
GPIO.setup(drive_R, GPIO.OUT)
GPIO.setup(drive_L, GPIO.OUT)
cameraServo_pwm = GPIO.PWM(cameraServo, 50) # GPIO 17 for PWM with 50Hz
drive_R_pwm = GPIO.PWM(drive_R, 50)
drive_L_pwm = GPIO.PWM(drive_L, 50)
cameraServo_pwm.start(0.0) # Initialization

drive_R_pwm.start(0.0) # Initialization
drive_L_pwm.start(0.0) # Initialization


# +++++++++++++++ Main loop ++++++++++++++++++++++++++++++++++++++++++++
while True:

  if(os.path.isfile('./commands.txt')):
    fileCommands=open("commands.txt", "r")
    if fileCommands.mode == 'r':
      contents = fileCommands.read()
      command_1 = contents[0:contents.find(" ")]
      command_2 = contents[contents.find(" "):len(contents)]
      print("command1=>", command_1, "< command2=>", command_2, "<\n")

      # Camera UP
      if(command_1 == 'c' and command_2.find("up")>-1):
        print("Moving camera UP\n")
        cameraServo_pwm.ChangeDutyCycle(5.0)
        sleep(1)
        cameraServo_pwm.ChangeDutyCycle(0.0)

      # Camera straight
      if(command_1 == 'c' and command_2.find("straight")>-1):
        print("Moving camera STRAIGHT\n")
        cameraServo_pwm.ChangeDutyCycle(7.5)
        sleep(1)
        cameraServo_pwm.ChangeDutyCycle(0.0)

      # Camera down
      if(command_1 == 'c' and command_2.find("down")>-1):
        print("Moving camera DOWN\n")
        cameraServo_pwm.ChangeDutyCycle(10.0)
        sleep(1)
        cameraServo_pwm.ChangeDutyCycle(0.0)


      # Drive forward
      if(command_1 == 'f'):
        driveTime = float(command_2) * 0.3
        motors(driveTime, driveTime, 0.1)
      # Drive backward
      if(command_1 == 'b'):
        driveTime = float(command_2) * 0.3
        motors(-driveTime, -driveTime, 0.1)
      # Turn left
      if(command_1 == 'tl'):
        driveTime = float(command_2) * 0.1
        motors(driveTime, -driveTime, 0.1)
      # Turn right
      if(command_1 == 'tr'):
        driveTime = float(command_2) * 0.1
        motors(-driveTime, driveTime, 0.1)
      # Drive to front left
      if(command_1 == 'fl'):
        driveTime = float(command_2) * 0.3
        motors(driveTime, driveTime/2.0, 0.5)
      # Drive to front right
      if(command_1 == 'fr'):
        driveTime = float(command_2) * 0.3
        motors(driveTime/2.0, driveTime, 0.5)
      # Drive to back left
      if(command_1 == 'br'):
        driveTime = float(command_2) * 0.3
        motors(-driveTime/2.0, -driveTime, 0.5)
      # Drive to back right
      if(command_1 == 'bl'):
        driveTime = float(command_2) * 0.3
        motors(-driveTime, -driveTime/2.0, 0.5)

      # Switch headlight
      if(command_1 == 'h'):
        if(int(command_2) == 0):
          led_1.off()
          led_2.off()
          led_3.on()
          led_4.off()
          led_5.off()
        if(int(command_2) == 1):
          led_1.on()
          led_2.on()
          led_3.on()
          led_4.off()
          led_5.off()
        if(int(command_2) == 2):
          led_1.off()
          led_2.off()
          led_3.on()
          led_4.on()
          led_5.on()
        if(int(command_2) == 3):
          led_1.on()
          led_2.on()
          led_3.on()
          led_4.on()
          led_5.on()

        if(int(command_2) == 4):
          led_1.on()
          led_2.on()
          led_3.off()
          led_4.on()
          led_5.on()

      # Delete commands.txt
      command_1 = ""
      command_2 = ""
      fileCommands.close
      print('returned value:', os.system("rm commands.txt"))

  # Write sensor data
  returned_value = os.system("pt-battery | grep Capacity > battery.txt")
  #print('returned value:', returned_value)

  bat_file = open("battery.txt","r")

  if bat_file.mode == 'r':
    contents = bat_file.read()

  bat_file.close

  sensors_file = open("sensors.txt","w+")
  if len(contents) > 10:
    sensors_file.write("Battery = % 7d%%" % int(contents[9:len(contents)-1]))
  sensors_file.write("\nlight =   % 7d" % light_sensor.reading)
  sensors_file.write("\ndistance =%7.2fcm\n" % ultrasonic_sensor.distance)
  sensors_file.close
  #returned_value = os.system("mv sensors.tmp sensors.txt")


  print("Loop done...\n")
  sleep(1)

